@@ -52,7 +52,6 @@
 #include "panel-applet-frame.h"
 #include "quick-desktop-reader.h"
 #include "xstuff.h"
-#include "egg-screen-exec.h"
 #include "panel-stock-icons.h"
 #include "panel-action-button.h"
 #include "panel-recent.h"
@@ -65,6 +64,7 @@
 #include "panel-globals.h"
 #include "panel-properties-dialog.h"
 #include "panel-run-dialog.h"
+#include "panel-lockdown.h"
 
 #undef MENU_DEBUG
 
@@ -131,7 +131,7 @@ static AppletMenuInfo action_buttons_info [] = {
 	{ N_("Force Quit"),
 	  N_("Actions"),
 	  "Actions",
-	  "Force a misbehaving application to quit",
+	  N_("Force a misbehaving application to quit"),
 	  NULL,
 	  PANEL_STOCK_FORCE_QUIT,
 	  PANEL_ACTION_FORCE_QUIT,
@@ -183,8 +183,7 @@ static GtkWidget * create_menu_at_fr (GtkWidget *menu,
 				      gboolean fake_submenus,
 				      gboolean force);
 
-static GtkWidget * create_desktop_menu (GtkWidget *m,
-					gboolean fake_sub);
+static GtkWidget * create_desktop_menu (GtkWidget *m);
 
 static void add_bonobo_applet (GtkWidget  *widget, const char *iid);
 
@@ -321,14 +320,17 @@ about_gnome_cb (GtkWidget *menuitem,
 		char      *program_path)
 {
 	GdkScreen *screen = menuitem_to_screen (menuitem);
+	GError    *error = NULL;
 
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 1, &program_path) < 0)
+	if (!gdk_spawn_command_line_on_screen (screen, program_path, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_about_gnome",
 				    _("Cannot execute '%s'"),
-				    _("%s - command not found"),
+				    _("%s: %s"),
 				    _("About GNOME"),
-				    program_path);
+				    program_path, error->message);
+		g_error_free (error);
+	}
 }
 
 static void
@@ -1419,13 +1421,18 @@ edit_direntry (GtkWidget    *widget,
 	
 }
 
-static void
-show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
+static gboolean
+show_item_menu (GtkWidget      *item,
+		GdkEventButton *bevent,
+		ShowItemMenu   *sim)
 {
 	GtkWidget        *menuitem;
 	GnomeDesktopItem *ii;
 	PanelWidget      *panel_widget;
 
+	if (panel_lockdown_get_locked_down ())
+		return FALSE;
+
 	panel_widget = menu_get_panel (sim->menuitem);
 
 	if (!sim->menu) {
@@ -1450,19 +1457,17 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 			if (ii == NULL) {
 				g_object_unref (G_OBJECT (sim->menu));
 				sim->menu = NULL;
-				return;
+				return TRUE;
 			}
 
-			if ( ! panel_toplevel_get_locked_down (panel_widget->toplevel)) {
-				menuitem = gtk_image_menu_item_new ();
-				setup_menuitem (menuitem, panel_menu_icon_get_size (),
-						NULL, _("Add this launcher to panel"), FALSE);
-				gtk_menu_shell_append (GTK_MENU_SHELL (sim->menu), menuitem);
-				g_signal_connect (menuitem, "activate",
-						  G_CALLBACK (add_app_to_panel),
-						  sim);
-				gtk_widget_set_sensitive (menuitem, objects_writable);
-			}
+			menuitem = gtk_image_menu_item_new ();
+			setup_menuitem (menuitem, panel_menu_icon_get_size (),
+					NULL, _("Add this launcher to panel"), FALSE);
+			gtk_menu_shell_append (GTK_MENU_SHELL (sim->menu), menuitem);
+			g_signal_connect (menuitem, "activate",
+					  G_CALLBACK (add_app_to_panel),
+					  sim);
+			gtk_widget_set_sensitive (menuitem, objects_writable);
 
 			menuitem = gtk_image_menu_item_new ();
 			setup_menuitem (menuitem, panel_menu_icon_get_size (),
@@ -1480,7 +1485,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 						   G_CALLBACK (gtk_menu_shell_deactivate),
 						   G_OBJECT (item->parent));
 
-			if ( ! panel_profile_get_inhibit_command_line ()) {
+			if (!panel_lockdown_get_disable_command_line ()) {
 				menuitem = gtk_image_menu_item_new ();
 				setup_menuitem (menuitem, panel_menu_icon_get_size (),
 						NULL, _("Put into run dialog"), FALSE);
@@ -1496,7 +1501,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 					 G_OBJECT(item->parent));
 			}
 
-			if ( ! panel_profile_get_inhibit_command_line ()) {
+			if (!panel_lockdown_get_disable_command_line ()) {
 				menuitem = gtk_image_menu_item_new ();
 				/*when activated we must pop down the first menu*/
 				g_signal_connect_swapped (G_OBJECT (menuitem),
@@ -1537,31 +1542,29 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 							   submenu);
 			}
 
-			if ( ! panel_toplevel_get_locked_down (panel_widget->toplevel)) {
-				menuitem = gtk_image_menu_item_new ();
-				setup_menuitem (menuitem, panel_menu_icon_get_size (), NULL,
-						_("Add this as drawer to panel"), FALSE);
-				gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
-				g_signal_connect (G_OBJECT(menuitem), "activate",
-						  G_CALLBACK(add_menudrawer_to_panel),
+			menuitem = gtk_image_menu_item_new ();
+			setup_menuitem (menuitem, panel_menu_icon_get_size (), NULL,
+					_("Add this as drawer to panel"), FALSE);
+			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+			g_signal_connect (G_OBJECT(menuitem), "activate",
+					  G_CALLBACK(add_menudrawer_to_panel),
 						  sim->mf);
-				gtk_widget_set_sensitive (menuitem,
-							  objects_writable &&
-							  toplevels_writable);
+			gtk_widget_set_sensitive (menuitem,
+						  objects_writable &&
+						  toplevels_writable);
 
-				menuitem = gtk_image_menu_item_new ();
-				setup_menuitem (menuitem, panel_menu_icon_get_size (),
-						NULL, _("Add this as menu to panel"), FALSE);
-				gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
-				g_signal_connect (G_OBJECT(menuitem), "activate",
-						  G_CALLBACK(add_menu_to_panel),
-						  sim->mf);
-				gtk_widget_set_sensitive (menuitem,
-							  objects_writable &&
-							  toplevels_writable);
-			}
+			menuitem = gtk_image_menu_item_new ();
+			setup_menuitem (menuitem, panel_menu_icon_get_size (),
+					NULL, _("Add this as menu to panel"), FALSE);
+			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+			g_signal_connect (G_OBJECT(menuitem), "activate",
+					  G_CALLBACK(add_menu_to_panel),
+					  sim->mf);
+			gtk_widget_set_sensitive (menuitem,
+						  objects_writable &&
+						  toplevels_writable);
 
-			if ( ! panel_profile_get_inhibit_command_line ()) {
+			if (!panel_lockdown_get_disable_command_line ()) {
 				menuitem = gtk_image_menu_item_new ();
 				setup_menuitem (menuitem, panel_menu_icon_get_size (),
 						NULL, _("Add new item to this menu"), FALSE);
@@ -1575,7 +1578,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 				g_signal_connect (menuitem, "activate",
 						  G_CALLBACK (add_new_app_to_menu),
 						  sim);
-				if ( ! panel_is_uri_writable (sim->mf->menudir))
+				if (!panel_is_uri_writable (sim->mf->menudir))
 					gtk_widget_set_sensitive (menuitem, FALSE);
 			}
 
@@ -1607,6 +1610,8 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 			item,
 			bevent->button,
 			bevent->time);
+
+	return TRUE;
 }
 
 gboolean
@@ -1625,14 +1630,8 @@ menuitem_button_press_event (GtkWidget      *menuitem,
 			     GdkEventButton *event,
 			     ShowItemMenu   *sim)
 {
-	/* no item menu in commie mode */
-	if (panel_profile_get_locked_down ())
-		return FALSE;
-	
-	if (event->button == 3) {
-		show_item_menu (menuitem, event, sim);
-		return TRUE;
-	}
+	if (event->button == 3)
+		return show_item_menu (menuitem, event, sim);
 	
 	return FALSE;
 }
@@ -1816,7 +1815,10 @@ setup_full_menuitem (GtkWidget   *menuitem,
 
 	GtkWidget *label;
 
-	label = gtk_label_new_with_mnemonic (title);
+	if (invisible_mnemonic)
+		label = gtk_label_new (title);
+	else
+		label = gtk_label_new_with_mnemonic (title);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
        
@@ -1854,7 +1856,7 @@ setup_full_menuitem (GtkWidget   *menuitem,
 				  G_CALLBACK (destroy_item_menu),
 				  sim);
 
-		if ( ! panel_profile_get_locked_down ()) {
+		if (!panel_lockdown_get_locked_down ()) {
 			gtk_drag_source_set(menuitem,
 					    GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 					    menu_item_targets, 1,
@@ -1888,8 +1890,7 @@ setup_applet_drag (GtkWidget *menuitem, const char *iid)
 		{ "application/x-panel-applet-iid", 0, 0 }
 	};
 	
-	if (iid == NULL ||
-	    panel_profile_get_locked_down ())
+	if (!iid || panel_lockdown_get_locked_down ())
 		return;
 	
 	gtk_drag_source_set (menuitem,
@@ -1914,8 +1915,7 @@ setup_internal_applet_drag (GtkWidget *menuitem, const char *applet_type)
 		{ "application/x-panel-applet-internal", 0, 0 }
 	};
 	
-	if (applet_type == NULL ||
-	    panel_profile_get_locked_down ())
+	if (!applet_type || panel_lockdown_get_locked_down ())
 		return;
 	
 	gtk_drag_source_set (menuitem,
@@ -2660,18 +2660,24 @@ append_internal_applets (GSList *list)
 {
 	static gboolean translated = FALSE;
 	int             i;
-	gboolean inhibit_command_line = panel_profile_get_inhibit_command_line ();
-	gboolean inhibit_force_quit = panel_profile_get_inhibit_force_quit ();
 
 	for (i = 0; i < G_N_ELEMENTS (action_buttons_info); i++) {
-		if (inhibit_command_line && 
+		if (panel_lockdown_get_disable_command_line () && 
 		    action_buttons_info [i].action_type == PANEL_ACTION_RUN)
 			continue;
 
-		if (inhibit_force_quit && 
+		if (panel_lockdown_get_disable_force_quit () && 
 		    action_buttons_info [i].action_type == PANEL_ACTION_FORCE_QUIT)
 			continue;
 
+		if (panel_lockdown_get_disable_lock_screen () && 
+		    action_buttons_info [i].action_type == PANEL_ACTION_LOCK)
+			continue;
+
+		if (panel_lockdown_get_disable_log_out () && 
+		    action_buttons_info [i].action_type == PANEL_ACTION_LOGOUT)
+			continue;
+
 		if (!translated) {
 			action_buttons_info [i].name = _(action_buttons_info [i].name);
 			action_buttons_info [i].category = _(action_buttons_info [i].category);
@@ -2681,6 +2687,8 @@ append_internal_applets (GSList *list)
                 list = g_slist_append (list, &action_buttons_info [i]);
         }
 
+	translated = TRUE;
+
         return list;
 }	
 
@@ -2695,7 +2703,6 @@ create_applets_menu (GtkWidget             *menu,
 	GSList            *langs_gslist;
 	GSList            *applets = NULL;
 	GSList		  *item = NULL;
-	GSList            *disabled_applets;
 	gboolean           applets_writable;
 	gboolean           objects_writable;
 
@@ -2719,8 +2726,6 @@ create_applets_menu (GtkWidget             *menu,
 		langs_glist = langs_glist->next;
 	}
 
-	disabled_applets = panel_profile_get_disabled_applets ();
-
 	for (i = 0; i < applet_list->_length; i++) {
 		Bonobo_ServerInfo *info;
 		AppletMenuInfo    *applet;
@@ -2732,7 +2737,7 @@ create_applets_menu (GtkWidget             *menu,
 
 		info = &applet_list->_buffer [i];
 
-		if (g_slist_find_custom (disabled_applets, info->iid, (GCompareFunc)strcmp) != NULL)
+		if (panel_lockdown_is_applet_disabled (info->iid))
 			continue;
 
 		name        = bonobo_server_info_prop_lookup (info, "name", langs_gslist);
@@ -2802,8 +2807,6 @@ create_applets_menu (GtkWidget             *menu,
 	g_slist_free (langs_gslist);
 	g_slist_free (applets);
 
-	panel_g_slist_deep_free	(disabled_applets);
-	
 	return menu;
 }
 
@@ -2935,21 +2938,18 @@ setup_remove_this_panel (GtkWidget *menu,
 {
 	PanelWidget *panel_widget;
 	GtkWidget   *label;
-	gboolean     toplevels_writable;
+	gboolean     sensitive;
 
 	panel_widget = menu_get_panel (menu);
 
 	g_assert (PANEL_IS_TOPLEVEL (panel_widget->toplevel));
 
-	toplevels_writable = panel_profile_list_is_writable (PANEL_GCONF_TOPLEVELS);
+	sensitive =
+		panel_toplevel_is_last_unattached (panel_widget->toplevel) ||
+		!panel_lockdown_get_locked_down () ||
+		!panel_profile_list_is_writable (PANEL_GCONF_TOPLEVELS);
 
-	if (panel_toplevel_is_last_unattached (panel_widget->toplevel) ||
-	    panel_toplevel_get_locked_down (panel_widget->toplevel))
-		gtk_widget_set_sensitive(menuitem, FALSE);
-	else
-		/* this can only be sensitive if we can actually write to
-		   the toplevels list */
-		gtk_widget_set_sensitive(menuitem, toplevels_writable);
+	gtk_widget_set_sensitive (menuitem, sensitive);
 
 	label = GTK_BIN(menuitem)->child;
 	if(GTK_IS_BOX(label)) {
@@ -2986,61 +2986,55 @@ make_panel_submenu (PanelWidget *panel_widget,
 	Bonobo_ServerInfoList *applet_list;
 	GtkWidget             *menuitem, *submenu;
 
-	if ( ! panel_toplevel_get_locked_down (panel_widget->toplevel)) {
-		menuitem = gtk_image_menu_item_new ();
-		setup_menuitem (menuitem,
-				GTK_ICON_SIZE_MENU,
-				gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU),
-				_("_Add to Panel"),
-				FALSE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	menuitem = gtk_image_menu_item_new ();
+	setup_menuitem (menuitem,
+			GTK_ICON_SIZE_MENU,
+			gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU),
+			_("_Add to Panel"),
+			FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
-		submenu = menu_new ();
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
-					   submenu);
+	submenu = menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
+				   submenu);
 
-		applet_list = instrument_add_submenu_for_reload (GTK_MENU_ITEM (menuitem), submenu);
+	applet_list = instrument_add_submenu_for_reload (GTK_MENU_ITEM (menuitem), submenu);
 
-		if ( ! panel_profile_list_is_writable (PANEL_GCONF_APPLETS) &&
-		     ! panel_profile_list_is_writable (PANEL_GCONF_OBJECTS)) {
-			/* if we can't write neither applets nor objects, just
-			   completely ignore this and make the whole submenu
-			   insensitive */
-			gtk_widget_set_sensitive (menuitem, FALSE);
-		} else {
-			make_add_submenu (submenu, applet_list);
-		}
+	if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS) ||
+	    panel_profile_list_is_writable (PANEL_GCONF_OBJECTS))
+		make_add_submenu (submenu, applet_list);
+	else
+		gtk_widget_set_sensitive (menuitem, FALSE);
 
-		menuitem = gtk_image_menu_item_new ();
+	menuitem = gtk_image_menu_item_new ();
 
-		setup_menuitem (menuitem, 
-				GTK_ICON_SIZE_MENU,
-				gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU),
-				_("_Delete This Panel"),
-				FALSE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		g_signal_connect (G_OBJECT (menuitem), "activate",
-				  G_CALLBACK (remove_panel),
-				  NULL);
-		g_signal_connect (G_OBJECT (menu), "show",
-				  G_CALLBACK(setup_remove_this_panel),
-				  menuitem);
+	setup_menuitem (menuitem, 
+			GTK_ICON_SIZE_MENU,
+			gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU),
+			_("_Delete This Panel"),
+			FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate",
+			  G_CALLBACK (remove_panel),
+			  NULL);
+	g_signal_connect (G_OBJECT (menu), "show",
+			  G_CALLBACK(setup_remove_this_panel),
+			  menuitem);
 
-		menuitem = gtk_image_menu_item_new ();
-		setup_menuitem (menuitem,
-				GTK_ICON_SIZE_MENU,
-				gtk_image_new_from_stock (
-							  GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU),
-				_("_Properties"),
-				FALSE);
+	menuitem = gtk_image_menu_item_new ();
+	setup_menuitem (menuitem,
+			GTK_ICON_SIZE_MENU,
+			gtk_image_new_from_stock (
+						  GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU),
+			_("_Properties"),
+			FALSE);
 
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		g_signal_connect_swapped (menuitem, "activate",
-					  G_CALLBACK (panel_properties_dialog_present), 
-					  panel_widget->toplevel);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect_swapped (menuitem, "activate",
+				  G_CALLBACK (panel_properties_dialog_present), 
+				  panel_widget->toplevel);
 
-		add_menu_separator (menu);
-	}
+	add_menu_separator (menu);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_menuitem (menuitem, 
@@ -3062,7 +3056,7 @@ static void
 show_panel_help (GtkWidget *w, gpointer data)
 {
 	panel_show_help (
-		gtk_widget_get_screen (w), "wgospanel.xml", NULL);
+		gtk_widget_get_screen (w), "user-guide.xml", NULL);
 }
 
 GtkWidget *
@@ -3074,7 +3068,7 @@ create_panel_context_menu (PanelWidget *panel)
 
 	retval = menu_new ();
 
-	if ( ! panel_profile_get_locked_down ())
+	if (!panel_lockdown_get_locked_down ())
 		make_panel_submenu (panel, retval);
 
 	menuitem = gtk_image_menu_item_new ();
@@ -3127,9 +3121,6 @@ ask_about_launcher_cb (GtkWidget *widget)
 	PanelData   *pd;
 	int          insertion_pos;
 
-	if (panel_profile_get_inhibit_command_line ())
-		return;
-
 	panel_widget = menu_get_panel (widget);
 
 	pd = g_object_get_data (G_OBJECT (panel_widget->toplevel), "PanelData");
@@ -3152,10 +3143,10 @@ make_add_submenu (GtkWidget             *menu,
 
 	create_applets_menu (menu, applet_list);
 
-	if ( ! panel_profile_get_inhibit_command_line ()) {
+	if (!panel_lockdown_get_disable_command_line ()) {
 		menuitem = gtk_image_menu_item_new ();
-		setup_stock_menu_item (
-				       menuitem, GTK_ICON_SIZE_MENU, PANEL_STOCK_LAUNCHER, _("Launcher..."), TRUE);
+		setup_stock_menu_item (menuitem, GTK_ICON_SIZE_MENU,
+				       PANEL_STOCK_LAUNCHER, _("Launcher..."), TRUE);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 		g_signal_connect (G_OBJECT(menuitem), "activate",
 				  G_CALLBACK(ask_about_launcher_cb),NULL);
@@ -3203,42 +3194,71 @@ make_add_submenu (GtkWidget             *menu,
 	gtk_widget_set_sensitive (menuitem, objects_writable);
 }
 
-static GtkWidget *
-create_desktop_menu (GtkWidget *menu, gboolean fake_submenus)
+static void
+append_lock_screen (GtkWidget *menu)
 {
-	GtkWidget *menuitem;
-	/* Panel entry */
+	GtkWidget  *menuitem;
 
-	if (menu == NULL) {
-		menu = menu_new ();
-	}
-
-	if (panel_is_program_in_path ("xscreensaver")) {
-		menuitem = gtk_image_menu_item_new ();
-		setup_stock_menu_item (
-			menuitem, panel_menu_icon_get_size (),
-			PANEL_STOCK_LOCKSCREEN, _("Lock Screen"), TRUE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		g_signal_connect (menuitem, "activate",
-				  G_CALLBACK (panel_action_lock_screen), NULL);
-		setup_internal_applet_drag(menuitem, "ACTION:lock:NEW");
-		gtk_tooltips_set_tip (panel_tooltips, menuitem,
-				      _("Lock the screen so that you can "
-					"temporarily leave your computer"), NULL);
-	}
+	if (panel_lockdown_get_disable_lock_screen () ||
+	    !panel_is_program_in_path ("xscreensaver"))
+		return;
 
 	menuitem = gtk_image_menu_item_new ();
-	setup_stock_menu_item (
-		menuitem, panel_menu_icon_get_size (), PANEL_STOCK_LOGOUT, _("Log Out"), TRUE);
+	setup_stock_menu_item (menuitem, panel_menu_icon_get_size (),
+			       PANEL_STOCK_LOCKSCREEN, _("Lock Screen"), TRUE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (menuitem, "activate",
+			  G_CALLBACK (panel_action_lock_screen), NULL);
+	setup_internal_applet_drag (menuitem, "ACTION:lock:NEW");
+	gtk_tooltips_set_tip (panel_tooltips, menuitem,
+			      _("Lock the screen so that you can "
+				"temporarily leave your computer"),
+			      NULL);
+}
+
+static void
+append_log_out (GtkWidget *menu)
+{
+	GtkWidget  *menuitem;
+	char       *logout_string;
+	char       *logout_tooltip;
+	const char *user_name;
+
+	if (panel_lockdown_get_disable_log_out ())
+		return;
+
+	menuitem = gtk_image_menu_item_new ();
+
+	logout_string = g_strdup_printf (_("Log Out %s"), g_get_user_name ());
+	setup_stock_menu_item (menuitem, panel_menu_icon_get_size (),
+			       PANEL_STOCK_LOGOUT, logout_string, TRUE);
+	g_free (logout_string);
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (panel_action_logout), 0);
-	setup_internal_applet_drag(menuitem, "ACTION:logout:NEW");
-	gtk_tooltips_set_tip (panel_tooltips, menuitem,
-			      _("Log out of this session to log in as "
-				"a different user or to shut down your "
-				"computer"),
-			      NULL);
+	setup_internal_applet_drag (menuitem, "ACTION:logout:NEW");
+
+	user_name = g_get_real_name ();
+	if (!user_name || !user_name [0])
+		user_name = g_get_user_name ();
+	
+	logout_tooltip = g_strdup_printf (_("Log out %s of this session to "
+					    "log in as a different user or to "
+					    "shut down your computer"),
+					  user_name);
+	gtk_tooltips_set_tip (panel_tooltips, menuitem, logout_tooltip, NULL);
+	g_free (logout_tooltip);
+}
+
+static GtkWidget *
+create_desktop_menu (GtkWidget *menu)
+{
+	if (!menu)
+		menu = menu_new ();
+
+	append_lock_screen (menu);
+	append_log_out (menu);
 
 	return menu;
 }
@@ -3338,7 +3358,7 @@ create_root_menu (GtkWidget   *root_menu,
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
 			menuitem, panel_menu_icon_get_size (),
-			PANEL_STOCK_GNOME_LOGO, _("Applications"), TRUE);
+			PANEL_STOCK_APPLICATIONS, _("Applications"), TRUE);
 		gtk_menu_shell_append (GTK_MENU_SHELL (root_menu), menuitem);
 		if(menu) {
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
@@ -3354,7 +3374,7 @@ create_root_menu (GtkWidget   *root_menu,
 					  FALSE /*launcher_add */);
 	}
 
-	if ( ! panel_profile_get_inhibit_command_line () && extra_items) {
+	if (!panel_lockdown_get_disable_command_line () && extra_items) {
 		add_menu_separator (root_menu);
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
@@ -3371,7 +3391,6 @@ create_root_menu (GtkWidget   *root_menu,
 
 	if (extra_items &&
 	    panel_is_program_in_path  ("gnome-search-tool")) {
-
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
 			menuitem, panel_menu_icon_get_size (),
@@ -3408,7 +3427,7 @@ create_root_menu (GtkWidget   *root_menu,
 		add_menu_separator (root_menu);
 	
 	if (flags & MAIN_MENU_DESKTOP_SUB) {
-		menu = create_desktop_menu (NULL, fake_submenus);
+		menu = create_desktop_menu (NULL);
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
 			menuitem, panel_menu_icon_get_size (),
@@ -3418,7 +3437,7 @@ create_root_menu (GtkWidget   *root_menu,
 	}
 
 	if (flags & MAIN_MENU_DESKTOP)
-		create_desktop_menu (root_menu, fake_submenus);
+		create_desktop_menu (root_menu);
 
 	return root_menu;
 }
@@ -3519,7 +3538,8 @@ panel_menu_key_press_handler (GtkWidget   *widget,
 {
 	gboolean retval = FALSE;
 
-	if (event->keyval == GDK_F10 && event->state == GDK_SHIFT_MASK) {
+	if (event->keyval == GDK_F10 &&
+	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_SHIFT_MASK) {
 		GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget);
 
 		retval = TRUE;
